@@ -18,6 +18,7 @@
 #define USE_UNSTABLE_LIBKDEGAMESPRIVATE_API
 #include <libkdegamesprivate/kgame/kplayer.h>
 // Qt
+#include <QApplication>
 #include <QColor>
 #include <QElapsedTimer>
 #include <QEvent>
@@ -190,21 +191,26 @@ void KWin4View::updateAndAdvance()
     if (mReflectionRect.width() > 0 && mReflectionRect.height() > 0) {
         // Draw reflection in steps to save processing power
         if (mReflectPhase == 0) {
-            mReflectImage = QImage(mReflectionRect.width(), mReflectionRect.height(), QImage::Format_ARGB32);
-            mReflectImage.fill(Qt::transparent);
-            QPainter imagePainter(&mReflectImage);
-            // imagePainter.fillRect(image.rect(),QBrush(Qt::red));
+            const qreal dpr = qApp->devicePixelRatio();
+            const int deviceWidth = mReflectionRect.width() * dpr;
+            const int deviceHeight = mReflectionRect.height() * dpr;
+            mReflectPixmap = QPixmap(deviceWidth, deviceHeight);
+            mReflectPixmap.fill(Qt::transparent);
+            QPainter pixmapPainter(&mReflectPixmap);
+            // pixmapPainter.fillRect(image.rect(),QBrush(Qt::red));
 
             // Turn on all optimizations
-            imagePainter.setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing | QPainter::SmoothPixmapTransform, false);
-            imagePainter.setClipping(true);
-            imagePainter.setWorldTransform(QTransform(1.0, 0.0, 0.0, -1.0, 0.0, mReflectImage.height()));
-            QRect source = QRect(mReflectionRect.x(), mReflectionRect.y() - mReflectImage.height(), mReflectImage.width(), mReflectImage.height());
+            pixmapPainter.setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing | QPainter::SmoothPixmapTransform, false);
+            pixmapPainter.setClipping(true);
+            pixmapPainter.setWorldTransform(QTransform(1.0, 0.0, 0.0, -1.0, 0.0, mReflectPixmap.height()));
+            const QRect source = QRect(mReflectionRect.x(), mReflectionRect.y() - mReflectionRect.height(), mReflectionRect.width(), mReflectionRect.height());
 
             bool vis = mReflectionSprite->isVisible();
             mReflectionSprite->hide();
             dynamic_cast<ReflectionGraphicsScene *>(scene())->setBackground(false);
-            scene()->render(&imagePainter, mReflectImage.rect(), source, Qt::IgnoreAspectRatio);
+            scene()->render(&pixmapPainter, mReflectPixmap.rect(), source, Qt::IgnoreAspectRatio);
+            mReflectPixmap.setDevicePixelRatio(dpr);
+
             dynamic_cast<ReflectionGraphicsScene *>(scene())->setBackground(true);
             if (vis)
                 mReflectionSprite->show();
@@ -213,17 +219,16 @@ void KWin4View::updateAndAdvance()
         // Draw reflection in steps to save processing power
         else if (mReflectPhase == 1) {
             // Semi transparent
-            QPainter imagePainter(&mReflectImage);
-            imagePainter.setTransform(QTransform());
-            imagePainter.setCompositionMode(QPainter::CompositionMode_DestinationIn);
-            imagePainter.drawImage(0, 0, mGradientImage);
+            QPainter pixmapPainter(&mReflectPixmap);
+            pixmapPainter.setTransform(QTransform());
+            pixmapPainter.setCompositionMode(QPainter::CompositionMode_DestinationIn);
+            pixmapPainter.drawPixmap(0, 0, mGradientPixmap);
             mReflectPhase = 2;
         }
         // Draw reflection in steps to save processing power
         else if (mReflectPhase == 2) {
             // Set to sprite
-            QPixmap pm = QPixmap::fromImage(mReflectImage);
-            mReflectionSprite->setPixmap(pm);
+            mReflectionSprite->setPixmap(mReflectPixmap);
             mReflectionSprite->update();
             mReflectPhase = 0;
         }
@@ -236,19 +241,24 @@ void KWin4View::setReflection(int x, int y, int width, int height)
 {
     mReflectionRect = QRect(x, y, width, height);
 
+    const qreal dpr = qApp->devicePixelRatio();
+    const int deviceWidth = width * dpr;
+    const int deviceHeight = height * dpr;
+
     QPoint p1, p2;
-    p2.setY(height);
+    p2.setY(deviceHeight);
     mGradient = QLinearGradient(p1, p2);
     mGradient.setColorAt(0, QColor(0, 0, 0, 100));
     mGradient.setColorAt(1, Qt::transparent);
 
     qCDebug(KFOURINLINE_LOG) << "Set reflection " << x << " " << y << " " << width << " " << height;
 
-    mGradientImage = QImage(width, height, QImage::Format_ARGB32);
-    mGradientImage.fill(Qt::transparent);
-    QPainter p(&mGradientImage);
-    p.fillRect(0, 0, width, height, mGradient);
+    mGradientPixmap = QPixmap(deviceWidth, deviceHeight);
+    mGradientPixmap.fill(Qt::transparent);
+    QPainter p(&mGradientPixmap);
+    p.fillRect(0, 0, deviceWidth, deviceHeight, mGradient);
     p.end();
+    mGradientPixmap.setDevicePixelRatio(dpr);
 
     mReflectionSprite->setPos(x, y);
     if (width > 0 && height > 0) {
